@@ -372,10 +372,11 @@ async fn handle_build_plan(
             } => {
                 let (p_out, p_conf) = translated_nodes[*parent].clone().unwrap();
                 let mut p_conf = (*p_conf).clone();
-                p_conf
+                let img_conf = p_conf
                     .config
-                    .get_or_insert_with(empty_image_config)
-                    .entrypoint = Some(new_entrypoint.to_owned());
+                    .get_or_insert_with(empty_image_config);
+                img_conf.entrypoint = Some(new_entrypoint.to_owned());
+                img_conf.cmd = None;
                 (p_out, Arc::new(p_conf))
             }
             SetLabel {
